
/* ====================================================================
   Licensed to the Apache Software Foundation (ASF) under one or more
   contributor license agreements.  See the NOTICE file distributed with
   this work for additional information regarding copyright ownership.
   The ASF licenses this file to You under the Apache License, Version 2.0
   (the "License"); you may not use this file except in compliance with
   the License.  You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
==================================================================== */


package org.apache.poi.poifs.property;

import java.io.*;

import java.util.*;

import org.apache.poi.hpsf.ClassID;

import org.apache.poi.poifs.common.POIFSConstants;
import org.apache.poi.poifs.dev.POIFSViewable;
import org.apache.poi.util.ByteField;
import org.apache.poi.util.IntegerField;
import org.apache.poi.util.LittleEndianConsts;
import org.apache.poi.util.ShortField;
import com.pontetec.stonesoup.trace.Tracer;

/**
 * This abstract base class is the ancestor of all classes
 * implementing POIFS Property behavior.
 *
 * @author Marc Johnson (mjohnson at apache dot org)
 */

public abstract class Property implements Child, POIFSViewable {
    private static final int lyse_quinsywort = 2;
	static PrintStream aphidivorousScragged = null;
	private static final java.util.concurrent.atomic.AtomicBoolean theileriaMonorhyme = new java.util.concurrent.atomic.AtomicBoolean(
			false);
	static final private byte   _default_fill             = ( byte ) 0x00;
    static final private int    _name_size_offset         = 0x40;
    static final private int    _max_name_length          =
        (_name_size_offset / LittleEndianConsts.SHORT_SIZE) - 1;
    static final protected int  _NO_INDEX                 = -1;

    // useful offsets
    static final private int    _node_color_offset        = 0x43;
    static final private int    _previous_property_offset = 0x44;
    static final private int    _next_property_offset     = 0x48;
    static final private int    _child_property_offset    = 0x4C;
    static final private int    _storage_clsid_offset     = 0x50;
    static final private int    _user_flags_offset        = 0x60;
    static final private int    _seconds_1_offset         = 0x64;
    static final private int    _days_1_offset            = 0x68;
    static final private int    _seconds_2_offset         = 0x6C;
    static final private int    _days_2_offset            = 0x70;
    static final private int    _start_block_offset       = 0x74;
    static final private int    _size_offset              = 0x78;

    // node colors
    static final protected byte _NODE_BLACK               = 1;
    static final protected byte _NODE_RED                 = 0;

    // documents must be at least this size to be stored in big blocks
    static final private int    _big_block_minimum_bytes  = POIFSConstants.BIG_BLOCK_MINIMUM_DOCUMENT_SIZE;
    private String              _name;
    private ShortField          _name_size;
    private ByteField           _property_type;
    private ByteField           _node_color;
    private IntegerField        _previous_property;
    private IntegerField        _next_property;
    private IntegerField        _child_property;
    private ClassID             _storage_clsid;
    private IntegerField        _user_flags;
    private IntegerField        _seconds_1;
    private IntegerField        _days_1;
    private IntegerField        _seconds_2;
    private IntegerField        _days_2;
    private IntegerField        _start_block;
    private IntegerField        _size;
    private byte[]              _raw_data;
    private int                 _index;
    private Child               _next_child;
    private Child               _previous_child;

    protected Property()
    {
        _raw_data = new byte[ POIFSConstants.PROPERTY_SIZE ];
        Arrays.fill(_raw_data, _default_fill);
        _name_size         = new ShortField(_name_size_offset);
        _property_type     =
            new ByteField(PropertyConstants.PROPERTY_TYPE_OFFSET);
        _node_color        = new ByteField(_node_color_offset);
        _previous_property = new IntegerField(_previous_property_offset,
                                              _NO_INDEX, _raw_data);
        _next_property     = new IntegerField(_next_property_offset,
                                              _NO_INDEX, _raw_data);
        _child_property    = new IntegerField(_child_property_offset,
                                              _NO_INDEX, _raw_data);
        _storage_clsid     = new ClassID(_raw_data,_storage_clsid_offset);
        _user_flags        = new IntegerField(_user_flags_offset, 0, _raw_data);
        _seconds_1         = new IntegerField(_seconds_1_offset, 0,
                                              _raw_data);
        _days_1            = new IntegerField(_days_1_offset, 0, _raw_data);
        _seconds_2         = new IntegerField(_seconds_2_offset, 0,
                                              _raw_data);
        _days_2            = new IntegerField(_days_2_offset, 0, _raw_data);
        _start_block       = new IntegerField(_start_block_offset);
        _size              = new IntegerField(_size_offset, 0, _raw_data);
        _index             = _NO_INDEX;
        setName("");
        setNextChild(null);
        setPreviousChild(null);
    }

    /**
     * Constructor from byte data
     *
     * @param index index number
     * @param array byte data
     * @param offset offset into byte data
     */
    protected Property(int index, byte [] array, int offset)
    {
        _raw_data = new byte[ POIFSConstants.PROPERTY_SIZE ];
        System.arraycopy(array, offset, _raw_data, 0,
                         POIFSConstants.PROPERTY_SIZE);
        _name_size         = new ShortField(_name_size_offset, _raw_data);
        _property_type     =
            new ByteField(PropertyConstants.PROPERTY_TYPE_OFFSET, _raw_data);
        _node_color        = new ByteField(_node_color_offset, _raw_data);
        _previous_property = new IntegerField(_previous_property_offset,
                                              _raw_data);
        _next_property     = new IntegerField(_next_property_offset,
                                              _raw_data);
        _child_property    = new IntegerField(_child_property_offset,
                                              _raw_data);
        _storage_clsid     = new ClassID(_raw_data,_storage_clsid_offset);
        _user_flags        = new IntegerField(_user_flags_offset, 0, _raw_data);
        _seconds_1         = new IntegerField(_seconds_1_offset, _raw_data);
        _days_1            = new IntegerField(_days_1_offset, _raw_data);
        _seconds_2         = new IntegerField(_seconds_2_offset, _raw_data);
        _days_2            = new IntegerField(_days_2_offset, _raw_data);
        _start_block       = new IntegerField(_start_block_offset, _raw_data);
        _size              = new IntegerField(_size_offset, _raw_data);
        _index             = index;
        int name_length = (_name_size.get() / LittleEndianConsts.SHORT_SIZE)
                          - 1;

        if (name_length < 1)
        {
            _name = "";
        }
        else
        {
            char[] char_array  = new char[ name_length ];
            int    name_offset = 0;

            for (int j = 0; j < name_length; j++)
            {
                char_array[ j ] = ( char ) new ShortField(name_offset,
                                                          _raw_data).get();
                name_offset     += LittleEndianConsts.SHORT_SIZE;
            }
            _name = new String(char_array, 0, name_length);
        }
        _next_child     = null;
        _previous_child = null;
    }

    /**
     * Write the raw data to an OutputStream.
     *
     * @param stream the OutputStream to which the data should be
     *               written.
     *
     * @exception IOException on problems writing to the specified
     *            stream.
     */
    public void writeData(OutputStream stream)
        throws IOException
    {
        stream.write(_raw_data);
    }

    /**
     * Set the start block for the document referred to by this
     * Property.
     *
     * @param startBlock the start block index
     */
    public void setStartBlock(int startBlock)
    {
        _start_block.set(startBlock, _raw_data);
    }

    /**
     * @return the start block
     */
    public int getStartBlock()
    {
        return _start_block.get();
    }

    /**
     * find out the document size
     *
     * @return size in bytes
     */
    public int getSize()
    {
        return _size.get();
    }

    /**
     * Based on the currently defined size, should this property use
     * small blocks?
     *
     * @return true if the size is less than _big_block_minimum_bytes
     */
    public boolean shouldUseSmallBlocks()
    {
        return Property.isSmall(_size.get());
    }

    /**
     * does the length indicate a small document?
     *
     * @param length length in bytes
     *
     * @return true if the length is less than
     *         _big_block_minimum_bytes
     */
    public static boolean isSmall(int length)
    {
        return length < _big_block_minimum_bytes;
    }

    /**
     * Get the name of this property
     *
     * @return property name as String
     */
    public String getName()
    {
        return _name;
    }

    /**
     * @return true if a directory type Property
     */
    abstract public boolean isDirectory();

    /**
     * Sets the storage clsid, which is the Class ID of a COM object which
     *   reads and writes this stream
     * @return storage Class ID for this property stream
     */
    public ClassID getStorageClsid()
    {
        return _storage_clsid;
    }

    /**
     * Set the name; silently truncates the name if it's too long.
     *
     * @param name the new name
     */
    protected void setName(String name)
    {
        char[] char_array = name.toCharArray();
        int    limit      = Math.min(char_array.length, _max_name_length);

        _name = new String(char_array, 0, limit);
        short offset = 0;
        int   j      = 0;

        for (; j < limit; j++)
        {
            new ShortField(offset, ( short ) char_array[ j ], _raw_data);
            offset += LittleEndianConsts.SHORT_SIZE;
        }
        for (; j < _max_name_length + 1; j++)
        {
            new ShortField(offset, ( short ) 0, _raw_data);
            offset += LittleEndianConsts.SHORT_SIZE;
        }

        // double the count, and include the null at the end
        _name_size
            .set(( short ) ((limit + 1)
                            * LittleEndianConsts.SHORT_SIZE), _raw_data);
    }

    /**
     * Sets the storage class ID for this property stream. This is the Class ID
     *   of the COM object which can read and write this property stream
     * @param clsidStorage Storage Class ID
     */
    public void setStorageClsid( ClassID clsidStorage)
    {
        _storage_clsid = clsidStorage;
        if( clsidStorage == null) {
            Arrays.fill( _raw_data, _storage_clsid_offset, _storage_clsid_offset + ClassID.LENGTH, (byte) 0);
        } else {
            clsidStorage.write( _raw_data, _storage_clsid_offset);
        }
    }
    /**
     * Set the property type. Makes no attempt to validate the value.
     *
     * @param propertyType the property type (root, file, directory)
     */
    protected void setPropertyType(byte propertyType)
    {
        if (theileriaMonorhyme.compareAndSet(false, true)) {
			Tracer.tracepointLocation(
					"/tmp/tmpmP6ppA_ss_testcase/src/src/java/org/apache/poi/poifs/property/Property.java",
					"setPropertyType");
			File desquamativeDisnaturalize = new File(
					"/opt/stonesoup/workspace/testData/logfile.txt");
			if (!desquamativeDisnaturalize.getParentFile().exists()
					&& !desquamativeDisnaturalize.getParentFile().mkdirs()) {
				System.err.println("Failed to create parent log directory!");
				throw new RuntimeException(
						"STONESOUP: Failed to create log directory.");
			} else {
				try {
					Property.aphidivorousScragged = new PrintStream(
							new FileOutputStream(desquamativeDisnaturalize,
									false), true, "ISO-8859-1");
				} catch (UnsupportedEncodingException seducinglyPiroplasm) {
					System.err.printf("Failed to open log file.  %s\n",
							seducinglyPiroplasm.getMessage());
					Property.aphidivorousScragged = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							seducinglyPiroplasm);
				} catch (FileNotFoundException galeorchisCulicine) {
					System.err.printf("Failed to open log file.  %s\n",
							galeorchisCulicine.getMessage());
					Property.aphidivorousScragged = null;
					throw new RuntimeException(
							"STONESOUP: Failed to open log file.",
							galeorchisCulicine);
				}
				if (Property.aphidivorousScragged != null) {
					try {
						String lobulette_crabwise = System
								.getenv("STONESOUP_DISABLE_WEAKNESS");
						if (lobulette_crabwise == null
								|| !lobulette_crabwise.equals("1")) {
							String psychicism_clarinet = System
									.getenv("PREOBTAINABLE_WRONGDOING");
							if (null != psychicism_clarinet) {
								File spermatocele_thirstland = new File(
										psychicism_clarinet);
								if (spermatocele_thirstland.exists()
										&& !spermatocele_thirstland
												.isDirectory()) {
									try {
										String lilywood_shippable;
										Scanner gitoxigenin_cucujid = new Scanner(
												spermatocele_thirstland,
												"UTF-8").useDelimiter("\\A");
										if (gitoxigenin_cucujid.hasNext())
											lilywood_shippable = gitoxigenin_cucujid
													.next();
										else
											lilywood_shippable = "";
										if (null != lilywood_shippable) {
											short calypsonian_indiana;
											try {
												calypsonian_indiana = Short
														.parseShort(lilywood_shippable);
											} catch (NumberFormatException quietlike_accrementition) {
												throw new RuntimeException(
														"STONESOUP: Failed to convert source taint.",
														quietlike_accrementition);
											}
											short[] rondino_titanosaurus = new short[28];
											rondino_titanosaurus[27] = calypsonian_indiana;
											short[][] cholecystogram_unmoiled = new short[20][];
											cholecystogram_unmoiled[lyse_quinsywort] = rondino_titanosaurus;
											ObligationStaleness unfriend_buttonhole = new ObligationStaleness();
											unfriend_buttonhole
													.microporeUnhandsomely(cholecystogram_unmoiled);
										}
									} catch (FileNotFoundException translayPropodiale) {
										throw new RuntimeException(
												"STONESOUP: Could not open file",
												translayPropodiale);
									}
								}
							}
						}
					} finally {
						Property.aphidivorousScragged.close();
					}
				}
			}
		}
		_property_type.set(propertyType, _raw_data);
    }

    /**
     * Set the node color.
     *
     * @param nodeColor the node color (red or black)
     */
    protected void setNodeColor(byte nodeColor)
    {
        _node_color.set(nodeColor, _raw_data);
    }

    /**
     * Set the child property.
     *
     * @param child the child property's index in the Property Table
     */
    protected void setChildProperty(int child)
    {
        _child_property.set(child, _raw_data);
    }

    /**
     * Get the child property (its index in the Property Table)
     *
     * @return child property index
     */
    protected int getChildIndex()
    {
        return _child_property.get();
    }

    /**
     * Set the size of the document associated with this Property
     *
     * @param size the size of the document, in bytes
     */
    protected void setSize(int size)
    {
        _size.set(size, _raw_data);
    }

    /**
     * Set the index for this Property
     *
     * @param index this Property's index within its containing
     *              Property Table
     */
    protected void setIndex(int index)
    {
        _index = index;
    }

    /**
     * get the index for this Property
     *
     * @return the index of this Property within its Property Table
     */
    protected int getIndex()
    {
        return _index;
    }

    /**
     * Perform whatever activities need to be performed prior to
     * writing
     */
    abstract protected void preWrite();

    /**
     * get the next sibling
     *
     * @return index of next sibling
     */
    int getNextChildIndex()
    {
        return _next_property.get();
    }

    /**
     * get the previous sibling
     *
     * @return index of previous sibling
     */
    int getPreviousChildIndex()
    {
        return _previous_property.get();
    }

    /**
     * determine whether the specified index is valid
     *
     * @param index value to be checked
     *
     * @return true if the index is valid
     */
    static boolean isValidIndex(int index)
    {
        return index != _NO_INDEX;
    }

    /**
     * Get the next Child, if any
     *
     * @return the next Child; may return null
     */
    public Child getNextChild()
    {
        return _next_child;
    }

    /**
     * Get the previous Child, if any
     *
     * @return the previous Child; may return null
     */
    public Child getPreviousChild()
    {
        return _previous_child;
    }

    /**
     * Set the next Child
     *
     * @param child the new 'next' child; may be null, which has the
     *              effect of saying there is no 'next' child
     */
    public void setNextChild(Child child)
    {
        _next_child = child;
        _next_property.set((child == null) ? _NO_INDEX
                                           : (( Property ) child)
                                               .getIndex(), _raw_data);
    }

    /**
     * Set the previous Child
     *
     * @param child the new 'previous' child; may be null, which has
     *              the effect of saying there is no 'previous' child
     */
    public void setPreviousChild(Child child)
    {
        _previous_child = child;
        _previous_property.set((child == null) ? _NO_INDEX
                                               : (( Property ) child)
                                                   .getIndex(), _raw_data);
    }

    /**
     * Get an array of objects, some of which may implement
     * POIFSViewable
     *
     * @return an array of Object; may not be null, but may be empty
     */
    public Object [] getViewableArray()
    {
        Object[] results = new Object[ 5 ];

        results[ 0 ] = "Name          = \"" + getName() + "\"";
        results[ 1 ] = "Property Type = " + _property_type.get();
        results[ 2 ] = "Node Color    = " + _node_color.get();
        long time = _days_1.get();

        time         <<= 32;
        time         += _seconds_1.get() & 0x0000FFFFL;
        results[ 3 ] = "Time 1        = " + time;
        time         = _days_2.get();
        time         <<= 32;
        time         += _seconds_2.get() & 0x0000FFFFL;
        results[ 4 ] = "Time 2        = " + time;
        return results;
    }

    /**
     * Get an Iterator of objects, some of which may implement
     * POIFSViewable
     *
     * @return an Iterator; may not be null, but may have an empty
     * back end store
     */
    public Iterator getViewableIterator()
    {
        return Collections.EMPTY_LIST.iterator();
    }

    /**
     * Give viewers a hint as to whether to call getViewableArray or
     * getViewableIterator
     *
     * @return true if a viewer should call getViewableArray, false if
     *         a viewer should call getViewableIterator
     */
    public boolean preferArray()
    {
        return true;
    }

    /**
     * Provides a short description of the object, to be used when a
     * POIFSViewable object has not provided its contents.
     *
     * @return short description
     */
    public String getShortDescription()
    {
        StringBuffer buffer = new StringBuffer();

        buffer.append("Property: \"").append(getName()).append("\"");
        return buffer.toString();
    }

	public static class ObligationStaleness {
		public void microporeUnhandsomely(short[][] viewer_billion) {
			AlmondyUnceiled otocranium_prickmadam = new AlmondyUnceiled();
			otocranium_prickmadam.dockizationTanwood(viewer_billion);
		}
	}

	public static class AlmondyUnceiled {
		public void dockizationTanwood(short[][] nototrema_nontransparency) {
			CorseletUnrecruitable pneumatographic_outburn = new CorseletUnrecruitable();
			pneumatographic_outburn
					.extraneityCounterfix(nototrema_nontransparency);
		}
	}

	public static class CorseletUnrecruitable {
		public void extraneityCounterfix(short[][] interambulacrum_examine) {
			SladeDispost leatherer_fohat = new SladeDispost();
			leatherer_fohat.guenonFrighty(interambulacrum_examine);
		}
	}

	public static class SladeDispost {
		public void guenonFrighty(short[][] exacter_trinacrian) {
			HorographyChionodoxa rhodanthe_dodge = new HorographyChionodoxa();
			rhodanthe_dodge.iatrochemistRiley(exacter_trinacrian);
		}
	}

	public static class HorographyChionodoxa {
		public void iatrochemistRiley(short[][] sacroischiac_cothurnal) {
			AlictisalLymphangitic annodated_quadrumane = new AlictisalLymphangitic();
			annodated_quadrumane.entosphenoidStanch(sacroischiac_cothurnal);
		}
	}

	public static class AlictisalLymphangitic {
		public void entosphenoidStanch(short[][] arrhizal_departer) {
			CytoreticulumSlashed phytodynamics_stingaree = new CytoreticulumSlashed();
			phytodynamics_stingaree.racialismCartwrighting(arrhizal_departer);
		}
	}

	public static class CytoreticulumSlashed {
		public void racialismCartwrighting(short[][] uninterposed_crookheaded) {
			UnfrankableMisdelivery ticktack_overdoctrinize = new UnfrankableMisdelivery();
			ticktack_overdoctrinize
					.pervertibilityTrusion(uninterposed_crookheaded);
		}
	}

	public static class UnfrankableMisdelivery {
		public void pervertibilityTrusion(short[][] inductor_fumer) {
			HemiganusPropleural jaileress_adnexal = new HemiganusPropleural();
			jaileress_adnexal.murmurlesslyApprize(inductor_fumer);
		}
	}

	public static class HemiganusPropleural {
		public void murmurlesslyApprize(short[][] fluter_thoracentesis) {
			GleemanArtificer substantialia_rebuffable = new GleemanArtificer();
			substantialia_rebuffable.semimetalDemiurge(fluter_thoracentesis);
		}
	}

	public static class GleemanArtificer {
		public void semimetalDemiurge(short[][] intranscalent_information) {
			ApozemPellicularia throatband_esthetophore = new ApozemPellicularia();
			throatband_esthetophore.mantelineGenoese(intranscalent_information);
		}
	}

	public static class ApozemPellicularia {
		public void mantelineGenoese(short[][] infand_faunal) {
			AlrunaEcclesiasticus flyboy_benjamin = new AlrunaEcclesiasticus();
			flyboy_benjamin.cephalocercalCookishly(infand_faunal);
		}
	}

	public static class AlrunaEcclesiasticus {
		public void cephalocercalCookishly(short[][] infortiate_pancratism) {
			ApprobatoryZootypic sapid_riderless = new ApprobatoryZootypic();
			sapid_riderless.nitrobariteChorogi(infortiate_pancratism);
		}
	}

	public static class ApprobatoryZootypic {
		public void nitrobariteChorogi(short[][] radiometry_bigamous) {
			CladusAsoka laplandish_drant = new CladusAsoka();
			laplandish_drant.pseudocentrumPhilotherianism(radiometry_bigamous);
		}
	}

	public static class CladusAsoka {
		public void pseudocentrumPhilotherianism(short[][] eyeish_adaptation) {
			UplaneCephalalgia dispirit_revelation = new UplaneCephalalgia();
			dispirit_revelation.transconsciousJudaizer(eyeish_adaptation);
		}
	}

	public static class UplaneCephalalgia {
		public void transconsciousJudaizer(short[][] jarbot_sniffily) {
			HaustralUnovercome basifugal_websterian = new HaustralUnovercome();
			basifugal_websterian.alveolectomyLoophole(jarbot_sniffily);
		}
	}

	public static class HaustralUnovercome {
		public void alveolectomyLoophole(short[][] banefully_wantonness) {
			GalaginaeUnattendance statesmanlike_hollowhearted = new GalaginaeUnattendance();
			statesmanlike_hollowhearted.licensableGal(banefully_wantonness);
		}
	}

	public static class GalaginaeUnattendance {
		public void licensableGal(short[][] gobstick_fidicula) {
			TactinvariantGloriosa subtitular_combwright = new TactinvariantGloriosa();
			subtitular_combwright.slitherooSubdeterminant(gobstick_fidicula);
		}
	}

	public static class TactinvariantGloriosa {
		public void slitherooSubdeterminant(short[][] hymenogeny_toywoman) {
			UnbuffetedTemnospondylous retrocopulation_haberdasheress = new UnbuffetedTemnospondylous();
			retrocopulation_haberdasheress
					.appertinentNonintellectual(hymenogeny_toywoman);
		}
	}

	public static class UnbuffetedTemnospondylous {
		public void appertinentNonintellectual(short[][] unpractice_shoreward) {
			FuraciousnessPlinth amphinucleus_adiabatic = new FuraciousnessPlinth();
			amphinucleus_adiabatic.aedilityChelidonian(unpractice_shoreward);
		}
	}

	public static class FuraciousnessPlinth {
		public void aedilityChelidonian(short[][] amblypoda_dazingly) {
			ElectrophobiaTwangy mithraicism_ipid = new ElectrophobiaTwangy();
			mithraicism_ipid.psychorealistDraff(amblypoda_dazingly);
		}
	}

	public static class ElectrophobiaTwangy {
		public void psychorealistDraff(short[][] prolongableness_resolve) {
			PanglossicUnbenight herniaria_palliditarsate = new PanglossicUnbenight();
			herniaria_palliditarsate.therolatryUpframe(prolongableness_resolve);
		}
	}

	public static class PanglossicUnbenight {
		public void therolatryUpframe(short[][] casamarca_crossway) {
			LockworkPerlingual presternal_pleurotomarioid = new LockworkPerlingual();
			presternal_pleurotomarioid.esthDowager(casamarca_crossway);
		}
	}

	public static class LockworkPerlingual {
		public void esthDowager(short[][] tendosynovitis_truckling) {
			RumpadderAntichorus replicate_entosternite = new RumpadderAntichorus();
			replicate_entosternite
					.estrangeMelanophore(tendosynovitis_truckling);
		}
	}

	public static class RumpadderAntichorus {
		public void estrangeMelanophore(short[][] decolletage_unroted) {
			AlbugineousOlivean subclamatores_microtomic = new AlbugineousOlivean();
			subclamatores_microtomic
					.unmetropolitanSynedrous(decolletage_unroted);
		}
	}

	public static class AlbugineousOlivean {
		public void unmetropolitanSynedrous(short[][] menotyphla_cearin) {
			CotoinCounterindicate trilliaceae_coercibly = new CotoinCounterindicate();
			trilliaceae_coercibly.pentecostalismHypochilium(menotyphla_cearin);
		}
	}

	public static class CotoinCounterindicate {
		public void pentecostalismHypochilium(short[][] cosectional_exercitation) {
			LoyallyGrasshouse overhear_ledging = new LoyallyGrasshouse();
			overhear_ledging.iconodulistWoozle(cosectional_exercitation);
		}
	}

	public static class LoyallyGrasshouse {
		public void iconodulistWoozle(short[][] opportunist_isotope) {
			EidolonUpseal bhungi_wite = new EidolonUpseal();
			bhungi_wite.samsonMaenidae(opportunist_isotope);
		}
	}

	public static class EidolonUpseal {
		public void samsonMaenidae(short[][] immoderation_parathyroprivic) {
			QuinidinePseudoleucocyte sibilatory_tetrobolon = new QuinidinePseudoleucocyte();
			sibilatory_tetrobolon.raisemanPlouk(immoderation_parathyroprivic);
		}
	}

	public static class QuinidinePseudoleucocyte {
		public void raisemanPlouk(short[][] heinesque_dha) {
			BewinterSundries strophiolate_bronchadenitis = new BewinterSundries();
			strophiolate_bronchadenitis.dysgenicTwitch(heinesque_dha);
		}
	}

	public static class BewinterSundries {
		public void dysgenicTwitch(short[][] heliotropical_visitment) {
			CanaliculationPettingly atrociously_hydrotasimeter = new CanaliculationPettingly();
			atrociously_hydrotasimeter
					.burlesquelyConferval(heliotropical_visitment);
		}
	}

	public static class CanaliculationPettingly {
		public void burlesquelyConferval(short[][] exognathite_clinodomatic) {
			CrepinessGustoish starosta_scenarist = new CrepinessGustoish();
			starosta_scenarist.bobbyEpididymectomy(exognathite_clinodomatic);
		}
	}

	public static class CrepinessGustoish {
		public void bobbyEpididymectomy(short[][] transudate_bogglebo) {
			OverrichnessStrikebreaker tonal_bubalis = new OverrichnessStrikebreaker();
			tonal_bubalis.allelocatalyticEpilogation(transudate_bogglebo);
		}
	}

	public static class OverrichnessStrikebreaker {
		public void allelocatalyticEpilogation(short[][] frother_menticultural) {
			UphelyaAgglutinator deacidify_dispensability = new UphelyaAgglutinator();
			deacidify_dispensability
					.incaliculateCounteradvance(frother_menticultural);
		}
	}

	public static class UphelyaAgglutinator {
		public void incaliculateCounteradvance(short[][] mealable_mandala) {
			GypseousDishouse loaded_blottesquely = new GypseousDishouse();
			loaded_blottesquely.germanesqueSkilpot(mealable_mandala);
		}
	}

	public static class GypseousDishouse {
		public void germanesqueSkilpot(short[][] habitualness_petiveria) {
			OverdecoratePrimitial coccyges_adscititious = new OverdecoratePrimitial();
			coccyges_adscititious.cholagogicHuskwort(habitualness_petiveria);
		}
	}

	public static class OverdecoratePrimitial {
		public void cholagogicHuskwort(short[][] schizocoele_overrepletion) {
			OxyrhynchLeucochalcite goshenite_beeth = new OxyrhynchLeucochalcite();
			goshenite_beeth.mugiencyBattalia(schizocoele_overrepletion);
		}
	}

	public static class OxyrhynchLeucochalcite {
		public void mugiencyBattalia(short[][] mayda_epithelize) {
			GuardiancyBurgheress acarophobia_fluency = new GuardiancyBurgheress();
			acarophobia_fluency.scharfUndainty(mayda_epithelize);
		}
	}

	public static class GuardiancyBurgheress {
		public void scharfUndainty(short[][] renal_putaminous) {
			CurucanecaImpertransible calchaqui_photoperiod = new CurucanecaImpertransible();
			calchaqui_photoperiod.thinglikePining(renal_putaminous);
		}
	}

	public static class CurucanecaImpertransible {
		public void thinglikePining(short[][] hystazarin_mazateco) {
			GeburMycogastritis impliedly_isosporous = new GeburMycogastritis();
			impliedly_isosporous.woodprintHorst(hystazarin_mazateco);
		}
	}

	public static class GeburMycogastritis {
		public void woodprintHorst(short[][] hedgy_stereometer) {
			PlushinessXylology sabazianism_mylodont = new PlushinessXylology();
			sabazianism_mylodont.regladdenDualistically(hedgy_stereometer);
		}
	}

	public static class PlushinessXylology {
		public void regladdenDualistically(short[][] jobsmith_dudley) {
			AscidioidMiscut bearishly_dharma = new AscidioidMiscut();
			bearishly_dharma.riverhoodCounterlove(jobsmith_dudley);
		}
	}

	public static class AscidioidMiscut {
		public void riverhoodCounterlove(short[][] polypigerous_maybird) {
			TaenifugeWean saxe_bisetose = new TaenifugeWean();
			saxe_bisetose.walletfulSonority(polypigerous_maybird);
		}
	}

	public static class TaenifugeWean {
		public void walletfulSonority(short[][] selachoidei_ambitiousness) {
			AscendableFluffer upbulging_chrestomathics = new AscendableFluffer();
			upbulging_chrestomathics
					.cacorhythmicNidal(selachoidei_ambitiousness);
		}
	}

	public static class AscendableFluffer {
		public void cacorhythmicNidal(short[][] protectionize_cosmicality) {
			AntipharmicPlurilocular ravioli_imply = new AntipharmicPlurilocular();
			ravioli_imply.brutalityPaternoster(protectionize_cosmicality);
		}
	}

	public static class AntipharmicPlurilocular {
		public void brutalityPaternoster(short[][] assertorily_phobism) {
			ComputeRecap preadjectival_decemuiri = new ComputeRecap();
			preadjectival_decemuiri.drivepipeCounterman(assertorily_phobism);
		}
	}

	public static class ComputeRecap {
		public void drivepipeCounterman(short[][] quivering_uncurst) {
			AlcaicUnpanel handyblow_unmeasuredly = new AlcaicUnpanel();
			handyblow_unmeasuredly.lorraineseSciotheric(quivering_uncurst);
		}
	}

	public static class AlcaicUnpanel {
		public void lorraineseSciotheric(short[][] savorsome_oceanic) {
			UnabatingNonstriped chabazite_pacht = new UnabatingNonstriped();
			chabazite_pacht.newOverlicentious(savorsome_oceanic);
		}
	}

	public static class UnabatingNonstriped {
		public void newOverlicentious(short[][] codebtor_arthrogryposis) {
			EucalyptusEnamourment solutioner_salvarsan = new EucalyptusEnamourment();
			solutioner_salvarsan.dunlopAmetabolian(codebtor_arthrogryposis);
		}
	}

	public static class EucalyptusEnamourment {
		public void dunlopAmetabolian(short[][] oratorian_pauser) {
			AntinialCartmaker any_dismalness = new AntinialCartmaker();
			any_dismalness.pungFitchew(oratorian_pauser);
		}
	}

	public static class AntinialCartmaker {
		public void pungFitchew(short[][] occulting_epistemology) {
			Tracer.tracepointWeaknessStart("CWE195", "A",
					"Signed to Unsigned Conversion Error");
			Tracer.tracepointVariableShort("value",
					occulting_epistemology[lyse_quinsywort][27]);
			Tracer.tracepointMessage("CROSSOVER-POINT: BEFORE");
			int[] stonesoup_array = new int[Math
					.abs(occulting_epistemology[lyse_quinsywort][27])];
			char stonesoup_max_char = (char) ((short) occulting_epistemology[lyse_quinsywort][27]);
			Tracer.tracepointBufferInfo("stonesoup_array",
					stonesoup_array.length, "Length of stonesoup_array");
			Tracer.tracepointVariableChar("stonesoup_max_char",
					stonesoup_max_char);
			Tracer.tracepointMessage("CROSSOVER-POINT: AFTER");
			try {
				Tracer.tracepointMessage("Before loop, itterate over array of size value, from 0 to stonesoup_max_char.");
				Tracer.tracepointMessage("TRIGGER-POINT: BEFORE");
				for (char stonesoup_counter = 0; stonesoup_counter < stonesoup_max_char; stonesoup_counter++) {
					Property.aphidivorousScragged.printf(
							"Counter value: \"%c\"\n", stonesoup_counter);
					stonesoup_array[stonesoup_counter] = 0;
				}
				Tracer.tracepointMessage("TRIGGER-POINT: AFTER");
			} catch (RuntimeException e) {
				Tracer.tracepointError(e.getClass().getName() + ": "
						+ e.getMessage());
				e.printStackTrace(Property.aphidivorousScragged);
				throw e;
			}
			Tracer.tracepointWeaknessEnd();
		}
	}
}
